local function insidePolygon(polygon, px, py)
    local oddNodes = false
    local j = #polygon - 1
    for i = 1, #polygon, 2 do
        if (polygon[i+1] < py and polygon[j+1] >= py or polygon[j+1] < py and polygon[i+1] >= py) then
          local coff = polygon[i+0] + (py - polygon[i+1]) / (polygon[j+1] - polygon[i+1]) * (polygon[j+0] - polygon[i+0])
            if coff < px then
              oddNodes = not oddNodes
            end
        end
        j = i;
    end
    return oddNodes
end

local debug_build = false
local main_character_id = "you"
local start_scene = "mainmenu" -- "mainmenu"

local characters = require "characters"

local sounds = {
  walk = { "sounds/walk-1.wav", "sounds/walk-2.wav" },
  click = { "sounds/doublelclick.wav" },
  scribble = { "sounds/scribble-1.wav" },
}

local cursors = {
  eye   = { "icons/mouse-eye.png",   x=31, y=36 },
  hand  = { "icons/mouse-hand.png",  x=31, y=27 },
  point = { "icons/mouse-point.png", x=13, y=13 },
  walk  = { "icons/mouse-walk.png",  x=28, y=46 },
  click = { "icons/mouse-click.png", x=26, y= 5 },
  talk  = { "icons/mouse-talk.png",  x=51, y=58 },
}

local fonts = { 
  dialog_name    = { "fonts/uwch.ttf", 30 },
  dialog_text    = { "fonts/TypewriterScribbled.ttf", 20 },
  hint           = { "fonts/Oswald-Regular.ttf", 12 },
  inventory_item = { "fonts/Oswald-Regular.ttf", 12 }, 
}

local images = {
  dialog_next = "icons/dialog-next.png",
  item_placeholder = "items/placeholder.png",
  blend_progress = "scenes/blendover.png",
}

local real_item_defs = {
  ["blue-key"] = { 
    text = "Blue Key",
    image = "items/blue-key.png",
  },
  ["red-key"] = { 
    text = "Red Key",
    image = "items/red-key.png",
  },
  ["bread"] = { 
    text = "Bread",
    image = "items/bread.png",
  },
  ["cheese"] = { 
    text = "Cheese",
    image = "items/cheese.png",
  },
  ["padlock-key"] = { 
    text = "Padlock Key",
    image = "items/padlock-key.png",
  },
  ["script"] = { 
    text = "The Script",
    image = "items/script.png",
  },
  ["lighter"] = { 
    text = "Lighter",
    image = "items/lighter.png",
  },
  ["zig-off"] = { 
    text = "Cigarette",
    image = "items/zig-off.png",
  },
  ["zig-on"] = { 
    text = "Burning Cigarette",
    image = "items/zig-on.png",
  },
  ["jigsaw"] = { 
    text = "Jigsaw",
    image = "items/jigsaw.png",
  },
  ["backdrop"] = { 
    text = "Backdrop",
    image = "items/backdrop.png",
  },
}

local all_scenes = {
  debug = "debug",
  alley = "alley",
  mainmenu = "mainmenu",
  backyard = "backyard",
  entryhall = "entryhall",
  lockerroom = "lockerroom",
  stage = "stage",
  theatre_front = "theatre-front",
  therapist = "therapist",
  requisites = "requisites",
  workshop = "workshop",
}

local all_dialogs = {
  demo = "demo",
  intro = "intro",
  ["aubrey-init"] = "aubrey-init",
  ["aubrey-script"] = "aubrey-script",
  ["aubrey-zig"] = "aubrey-zig",
  ["aubrey-fire-zig"] = "aubrey-fire-zig",
  ["zachary"] = "zachary",
  ["accident-aubrey"] = "accident-aubrey",
}

local function makeButton()
  return {
    last = false,
    up = false,
    down = false,
  }
end

local function updateButton(button, new)
  if button.last ~= new then
    button.up   = not new
    button.down = new
    button.last = new
  else
    button.up = false
    button.down = false
  end
end

local next_scene = nil
local next_dialog = nil

local game_api = { }
setmetatable(game_api, { __index = _G })




function combineItems(state, item_a, item_b)

  if item_a == "lighter" and item_b == "zig-off" then
    state:removeItem("zig-off")
    state:giveItem("zig-on")
  end

end




local scene

local state = {
  inventory = {
    mood = 0,
  },
  mood = "bright", -- "bright" or "dark"
}

local function loadScene(id)
  local path = string.format("scenes/%s", id)

  local scene_loader = assert(love.filesystem.load(path .. "/data.lua"))
  debug.setfenv(scene_loader, game_api)
  local scene = scene_loader()

  scene.bright_bg = love.graphics.newImage(path .. "/bright.png")
  scene.dark_bg = love.graphics.newImage(path .. "/dark.png")

  for i=1,#scene.zones do
    local zone = scene.zones[i]

    if zone.visual then
      zone.visual[1] = love.graphics.newImage(zone.visual[1])
      zone.visual.x = zone.visual.x or 0
      zone.visual.y = zone.visual.y or 0
    end
  end

  return scene
end

local function loadDialog(id)
  
  local dialog = {
    api = { },
  }
  
  setmetatable(dialog.api, { __index = game_api }) -- "derive from" game api
  function dialog.api.makeFoo()
    
  end
  
  if type(id) == "string" then
    local path = string.format("dialogs/%s.lua", id)
    local dialog_loader = love.filesystem.load(path)
    debug.setfenv(dialog_loader, dialog.api)
    dialog.data = dialog_loader()
  elseif type(id) == "table" then
    dialog.data = id 
  else
    error("unsupported type")
  end
  assert(type(dialog.data) == "table")

  for i=1,#dialog.data do
    local node = dialog.data[i]
    node.who = node.who or main_character_id
  end

  function dialog:reset()
    self.index = 1
    self.active_spot = nil
  end

  function dialog:isDone()
    return self.index > #self.data
  end

  function dialog:getNode()
    return self.data[self.index]
  end

  function dialog:advance(selected_line)
    local node = self.data[self.index]

    state:evaluate(node.after)

    local next_item = self.index + 1
    local next = state:evaluate(node.next)
    if next then
      next_item = next
    end

    if node.kind == "choice" then
      local next = state:evaluate(node[selected_line].next) 
      if next then
        next_item = next
      end
    end

    if next_item == "$" then
      self.index = #self.data + 1
    elseif type(next_item) == "string" then
      local n = nil
      for i=1,#self.data do
        if self.data[i].label == next_item then
          n = i
          break
        end
      end
      assert(n, "Unknown label '" .. next_item .. "'")
      self.index = n
    elseif type(next_item) == "number" then
      self.index = next_item
    end
  end

  dialog:reset()

  return dialog
end

local keyboard = {
  escape = makeButton(),
  space = makeButton(),
  f6 = makeButton(), -- all items
  f7 = makeButton(), -- darken mood
  f8 = makeButton(), -- brighten mood
  f9 = makeButton(), -- scene debugger
  f10 = makeButton(), -- editor
  f11 = makeButton(), -- inventory debugger
}
local mouse_left = makeButton()
local mouse_right = makeButton()
local mouse_middle = makeButton()

local editor_polygon = { }


function state:updateMood()
  local cnt = self:itemCount("mood")
  if cnt > 1 then
    self.mood = "bright"
    self.inventory.mood = 1
  elseif cnt < -1 then
    self.mood = "dark"
    self.inventory.mood = -1
  end
end

function state:itemCount(item)
  return self.inventory[item] or 0
end

function state:hasItem(item)
  return (self.inventory[item] or 0) > 0
end

function state:giveItem(item, count)
  count = count or 1 
  self.inventory[item] = (self.inventory[item] or 0) + count

  self:updateMood()
end

function state:setItemCount(item, count)
  self.inventory[item] = count or 1 
  self:updateMood()
end

function state:removeItem(item, count)
  count = count or 1 
  local c = (self.inventory[item] or 0) - count

  -- we cannot have less than zero items in the inventory,
  -- but for game logic items, we can!
  if real_item_defs[item] then
    if c < 0 then c = 0 end
  end

  self.inventory[item] = c

  self:updateMood()
end

function state:evaluate(value, ...)
  if type(value) == "function" then 
    return value(self, ...)
  else
    return value
  end
end

local blend_background_shader 

function love.load()
  love.window.setMode(1280, 720)

  blend_background_shader = love.graphics.newShader [[

    extern Image blend_mask;
    extern float blend_progress;

    vec4 effect( vec4 color, Image tex, vec2 texture_coords, vec2 screen_coords )
    {
      float alpha = Texel(blend_mask, texture_coords).r;

      vec4 texturecolor = Texel(tex, texture_coords);
      vec4 final = texturecolor * color;
      final.a *= step(blend_progress, alpha);
      return final;
    }
  ]]


  for cursor,config in pairs(cursors) do
    -- local img = love.image.newImageData(config[1])
    -- local cur = love.mouse.newCursor(img, config.x, config.y)
    config[1] = love.graphics.newImage(config[1])
    -- cursors[cursor] = cur
  end

  for item, def in pairs(real_item_defs) do
    def.image = love.graphics.newImage(def.image)
  end

  for name, path in pairs(images) do
    images[name] = love.graphics.newImage(path)
  end

  for name in pairs(all_scenes) do 
    all_scenes[name] = loadScene(name)
  end

  for name in pairs(all_dialogs) do 
    all_dialogs[name] = loadDialog(name)
  end

  for name, config in pairs(fonts) do 
    fonts[name] = love.graphics.newFont(config[1], config[2])
  end

  for name, soundfx in pairs(sounds) do

    for i=1,#soundfx do
      soundfx[i] = love.audio.newSource(soundfx[i], "static")
    end

    function soundfx:play()
      local i = math.random(#self)
      local s = self[i]
      s:stop()
      s:play()
    end

  end

  for name, char in pairs(characters) do
    if char.portrait ~= nil then
      char.portrait = love.graphics.newImage(char.portrait)
    end
  end

  scene = all_scenes[start_scene]
end

local in_edit_mode = false
local inventory_debug_mode = false
local stage_debug_mode = false

local current_mood_level = 1.0

local dragged_item = nil
local hovered_inventory_item = nil
local current_polygon = nil
local current_dialog = nil
function love.update(dt) --
  local mx, my = love.mouse.getPosition()

  if love.keyboard.isDown("escape") then 
    game_api.setScene("mainmenu", "none")
    current_dialog = nil
  end

  for k,v in pairs(keyboard) do
    updateButton(v, love.keyboard.isDown(k))
  end

  updateButton(mouse_left, love.mouse.isDown(1))
  updateButton(mouse_right, love.mouse.isDown(2))
  updateButton(mouse_middle, love.mouse.isDown(3))

  if next_scene ~= nil then
    -- print("change from", scene, "to", next_scene)
    scene = next_scene[1]

    if next_scene[2] ~= "none" then
      local fx = sounds[next_scene[2]] or sounds.walk
      fx:play()
    end

    next_scene = nil
  end

  if next_dialog ~= nil then
    current_dialog = next_dialog
    current_dialog:reset()
    next_dialog = nil
    dragged_item = nil
    hovered_inventory_item = nil
  end

  if keyboard.f10.down then
    in_edit_mode = not in_edit_mode
  end

  if keyboard.f11.down then
    inventory_debug_mode = not inventory_debug_mode
  end

  if keyboard.f9.down then
    stage_debug_mode = not stage_debug_mode
  end

  if keyboard.f7.down then
    state.inventory.mood = -10
    state:updateMood()
  end
  if keyboard.f8.down then
    state.inventory.mood = 10
    state:updateMood()
  end
  if keyboard.f6.down then
    for item in pairs(real_item_defs) do 
      state.inventory[item] = 10
    end
  end
  
  current_polygon = nil
  if in_edit_mode then

    if mouse_left.down then
      table.insert(editor_polygon, mx)
      table.insert(editor_polygon, my)
    end
    if mouse_right.down and #editor_polygon > 0 then
      editor_polygon[#editor_polygon] = nil
      editor_polygon[#editor_polygon] = nil
    end
    if mouse_middle.down and #editor_polygon >= 6 then

      io.stdout:write("{\n")
      io.stdout:write("  cursor = \"eye\",\n")
      io.stdout:write("  visible = true,\n")
      io.stdout:write("  click = nil,\n")
      io.stdout:write("  hint = nil,\n")
      io.stdout:write("  ")
      for i=1,#editor_polygon do
        io.stdout:write(("%d, "):format(editor_polygon[i]))
      end
      io.stdout:write("\n")
      io.stdout:write("},\n")

      table.insert(scene.zones, editor_polygon)
      editor_polygon = { }
    end

  elseif current_dialog ~= nil then
      local node = current_dialog:getNode()

    if mouse_left.down then

      if node.kind == "choice" then
        if current_dialog.active_spot then
          current_dialog:advance(current_dialog.active_spot)
        end
      else
        current_dialog:advance()
      end
    end

    if current_dialog:isDone() then
      current_dialog = nil
    end

  else
    for i=#scene.zones,1,-1 do
      local polygon = scene.zones[i]

      local vis = state:evaluate(polygon.visible)
      if vis and insidePolygon(polygon, mx, my) then
        current_polygon = polygon
        break
      end
    end

    if current_polygon ~= nil then
      local dx, dy = 0, 0

      if love.keyboard.isDown("left")  then dx = dx - 1 end
      if love.keyboard.isDown("right") then dx = dx + 1 end
      if love.keyboard.isDown("up")    then dy = dy - 1 end
      if love.keyboard.isDown("down")  then dy = dy + 1 end

      if dx ~= 0 or dy ~= 0 then

        if love.keyboard.isDown("lshift") then
          current_polygon.visual.x = current_polygon.visual.x + dx
          current_polygon.visual.y = current_polygon.visual.y + dy
          io.stdout:write(("x=%d, y=%d"):format(current_polygon.visual.x, current_polygon.visual.y))
        else
          for i=1,#current_polygon do
            local v = current_polygon[i]
            if i%2 == 1 then
              v = v + dx
            else
              v = v + dy
            end

            current_polygon[i] = v
          end
          for i=1,#current_polygon do
            io.stdout:write(("%d, "):format(current_polygon[i]))
          end
        end
        io.stdout:write("\n")
      end

    end

    if mouse_left.down then
      local item = nil
      if dragged_item ~= nil then
        item = dragged_item.item
      end
      dragged_item = nil
      if hovered_inventory_item ~= nil then

        if item ~= nil then
          local tab = { hovered_inventory_item.item, item }
          table.sort(tab)

          combineItems(state, tab[1], tab[2])
        else
          dragged_item = hovered_inventory_item
        end
      elseif current_polygon ~= nil and current_polygon.click ~= nil then        
        current_polygon.click(state, item)
      end
    end

  end

  local cur = nil
  if current_polygon ~= nil then
    cur = cursors[state:evaluate(current_polygon.cursor)]
  end

  current_hack_cursor = cur or cursors.point
  -- love.mouse.setCursor(cur or cursors.point)
end

function drawPolygonHighlighted(polygon, highlight)
  if not debug_build then
    return 
  end
  if highlight then
    love.graphics.setColor(1, 0, 0, 0.5)
  else
    love.graphics.setColor(0.5, 0.5, 0.5, 0.3)
  end
  love.graphics.polygon("fill", polygon)
end

function love.draw(...)

    local sw, sh = love.graphics.getDimensions()
    local mx, my = love.mouse.getPosition()

    love.graphics.reset()
    love.graphics.clear(0, 0, 0)

    local theme = state:evaluate(scene.theme) or state.mood

    do
      local target_mood_level = 0.0
      if state.mood == "bright" then
        target_mood_level = 1.0
      end

      local dir = 0
      if current_mood_level > target_mood_level then
        dir = -1
      elseif current_mood_level < target_mood_level then
        dir = 1
      end

      current_mood_level = current_mood_level + 1.4 * dir * love.timer.getDelta()
      if current_mood_level < 0 then
        current_mood_level = 0
      end
      if current_mood_level > 1 then
        current_mood_level = 1
      end

    end

    love.graphics.setColor(1, 1, 1, 1)
    love.graphics.draw(scene.bright_bg, 0, 0)
    love.graphics.setShader(blend_background_shader)
    if blend_background_shader:hasUniform("blend_mask") then blend_background_shader:send("blend_mask", images.blend_progress) end 
    if blend_background_shader:hasUniform("blend_progress") then blend_background_shader:send("blend_progress", current_mood_level) end 
    love.graphics.draw(scene.dark_bg, 0, 0)
    love.graphics.setShader()

    -- love.graphics.setColor(0.5, 0.5, 0.5, 1)
    -- love.graphics.print(tostring(current_mood_level), 10, 10)

    for i=1,#scene.zones do
      local polygon = scene.zones[i]
      local vis = state:evaluate(polygon.visible)
      if vis then
        if polygon.visual then
          love.graphics.setColor(1, 1, 1, 1)
          love.graphics.draw(
            polygon.visual[1],
            polygon[1] +  polygon.visual.x,
            polygon[2] +  polygon.visual.y
          )
        end
        drawPolygonHighlighted(polygon, polygon == current_polygon)
      end
    end

    do
      love.graphics.setColor(1, 1, 1)
      for i=1,#editor_polygon,2 do
        local x = editor_polygon[i+0]
        local y = editor_polygon[i+1]
        love.graphics.line(x - 3, y - 3, x + 3, y + 3)
        love.graphics.line(x - 3, y + 3, x + 3, y - 3)
      end
      if #editor_polygon >= 4 then
        love.graphics.line(editor_polygon)
      end
      if #editor_polygon >= 6 then
        drawPolygonHighlighted(editor_polygon, true)
      end
    end

    do
      local lh = fonts.inventory_item:getHeight()
      love.graphics.setFont(fonts.inventory_item)

      local item_size = 32

      local function getDisplay(item, count)
        local real = real_item_defs[item]
        if not real then
          if inventory_debug_mode then
            return images.item_placeholder, item
          else
            return nil
          end
        end
        if count <= 0 then
          return nil
        end
        return real.image, real.text
      end
      
      local count = 0
      for item,c in pairs(state.inventory) do
        if getDisplay(item, c) ~= nil then
          count = count + 1
        end
      end
      
      hovered_inventory_item = nil

      if count > 0 then
        local padding = 10
        local margin = 50

        local width = 200
        local height = item_size * count + 2 * padding
        local left = sw - width - margin
        local top = margin

        love.graphics.setColor(1, 1, 1, 0.8)
        love.graphics.rectangle("fill", left, top, width, height)

        love.graphics.setColor(0, 0, 0)
        love.graphics.setLineWidth(2.0)
        love.graphics.rectangle("line", left, top, width, height)

        local y = top + padding
        for item,count in pairs(state.inventory) do
          local image, text = getDisplay(item, count)
          if image ~= nil then

            local x, w, h = left + padding, width - 2 * padding, item_size

            local focus = mx >= x and mx < x + w and my >= y and my < y + h

            if focus and current_dialog == nil then

              local real = real_item_defs[item]

              if real then 
                hovered_inventory_item = {
                  item = item,
                  real = real,
                }
                love.graphics.setColor(1,1,1,0.5)
                love.graphics.rectangle("fill", x, y, w, h)
              end
            end

            love.graphics.setColor(1,1,1)
            love.graphics.draw(image, left + padding, y)
            love.graphics.setColor(0, 0, 0)
            love.graphics.printf(text, x + padding + item_size, y + (item_size - lh) / 2, width - 2 * item_size, "left")
            love.graphics.printf(tostring(count), x + padding + item_size, y + (item_size - lh) / 2, width - 2 * item_size, "right")
            y = y + item_size
          end
        end
      end
    end

    if current_polygon ~= nil then

      local text = state:evaluate(current_polygon.hint)

      if text then

        local width = fonts.hint:getWidth(text)
        local lh = fonts.hint:getHeight()
        love.graphics.setFont(fonts.hint)

        local x, y, w, h = mx + lh, my - 3.0 * lh, lh + width, 2 * lh

        if x + w >= sw then
          x = sw - w
        end
        if y < 0 then
          y = 0
        end

        love.graphics.setColor(0.5, 0.5, 0.5, 0.8)
        love.graphics.rectangle("fill", x, y, w, h)

        love.graphics.setColor(0, 0, 0)
        love.graphics.print(text, x + 0.5 * lh, y + 0.5 * lh)
      end

    end

    if current_dialog ~= nil then

      local text_padding = 20
      local name_padding = 10
      local margin = 50

      
      local node = current_dialog:getNode()

      local char = characters[node.who]
      local display_name = char.name
      local portrait = char.portrait

      local name_height = fonts.dialog_name:getHeight()
      local name_width = fonts.dialog_name:getWidth(display_name) + 2 * text_padding


      local line_height = fonts.dialog_text:getHeight()
      
      local height = 4 * line_height + 2 * text_padding

      local top = sh - height - margin

      if portrait then
        love.graphics.setColor(1,1,1)
        love.graphics.draw(portrait, sw - margin - portrait:getWidth(), top - char.portrait_offset)
      end

      love.graphics.setColor(1, 1, 1, 0.8)
      love.graphics.rectangle("fill", margin, top, sw - 2 * margin, height)
      
      current_dialog.active_spot = nil
      
      love.graphics.setFont(fonts.dialog_text)
      love.graphics.setLineWidth(1.0)
      for i=1, #node do
        local x, y, w, h = text_padding + margin, top + text_padding + line_height * (i - 1), sw - 2 * (text_padding + margin), line_height

        local text = node[i]
        if type(text) == "table" then
          text = text[1]

          -- Only "table" choices are allowed to be selected
          if node.kind == "choice" then
            local active = mx >= x and mx < x + w and my >= y and my < y + h
            
            if active then
              current_dialog.active_spot = i

              love.graphics.setColor(1, 1, 1, 0.8)
              love.graphics.rectangle("fill", x, y, w, h)
            end
          end
        end

        love.graphics.setColor(0, 0, 0)
        love.graphics.print(text, x, y)
      end

      love.graphics.setColor(1, 1, 1, 0.8)

      love.graphics.rectangle("fill", margin, top - name_height - 2 * name_padding, name_width, name_height + 2 * name_padding)

      love.graphics.setFont(fonts.dialog_name)
      love.graphics.setColor(0, 0, 0)
      love.graphics.print(display_name, margin + text_padding, top - name_height - name_padding)

      love.graphics.setColor(0, 0, 0)
      love.graphics.setLineWidth(2.0)
      
      love.graphics.rectangle("line", margin, sh - height - margin, sw - 2 * margin, height)
      love.graphics.rectangle("line", margin, sh - height - name_height - margin - text_padding, name_width, name_height + text_padding)

      if node.kind ~= "choice"  then
        -- local blinky = love.timer.getTime() % 0.7

        local t = love.timer.getTime()

        local dy = math.floor( math.abs(5 * math.sin(1.4 * t)))
        
        -- if blinky > 0.3 then
          love.graphics.setColor(0, 0, 0)
          love.graphics.draw(images.dialog_next, sw - margin - images.dialog_next:getWidth() - 2, sh - margin - images.dialog_next:getHeight() - 2 + dy)
        -- end
      end

    end

    if dragged_item ~= nil then 
      local real = assert(dragged_item.real)

      love.graphics.setColor(1, 1, 1)
      love.graphics.draw(real.image, mx - real.image:getWidth() / 2, my - real.image:getHeight() / 2)
    end

    if in_edit_mode then
      love.graphics.setColor(0, 0, 0)
      love.graphics.print("EDIT MODE", 10, 10)
    end
    

    if stage_debug_mode then
      love.graphics.setColor(1, 0, 0)

      local top = 10
      for name, sc in pairs(all_scenes) do
        love.graphics.print(name, 10, top)
        
        if mouse_left.down and mx < 100 and my >= top and my < top + 20 then
          next_scene = { sc, "none" }
        end
        
        top = top + 20
      end
      
    end
    

    love.graphics.setColor(1, 1, 1)

    love.mouse.setCursor()
    love.graphics.draw(
      current_hack_cursor[1],
      mx - current_hack_cursor.x,
      my - current_hack_cursor.y
    )

    -- love.graphics.line(mx - 10, my - 10, mx + 10, my + 10)
    -- love.graphics.line(mx - 10, my + 10, mx + 10, my - 10)

end














function game_api.setScene(scene, effect)
  local scene = all_scenes[ assert(tostring(scene), "scene name must be given!") ]
  if scene then
    next_scene = { scene, effect }
  end
end

function game_api.startDialog(dialog)
  assert(dialog)
  if type(dialog) == "string" then
    next_dialog = assert(all_dialogs[dialog], "unknown dialog " .. dialog)
  elseif type(dialog) == "table" then
    next_dialog = dialog
  end
end

function game_api.getTime(...)
  return love.timer.getTime()
end

function game_api.playSound(name)
  sounds[name]:play()
end

function game_api.createDialog(data)
  return loadDialog(data)
end

function game_api.createOneLiner(line, who)
  local node = { who = who }
  for str in string.gmatch(line, "([^\n]+)") do
    table.insert(node, str)
  end
  return loadDialog { node }
end