local function doneState(state)
  local practise_done = state:hasItem("task-practise-done")
  local backdrop_done = state:hasItem("task-backdrop-done")
  local lighting_done = true -- state:hasItem("task-lighting-done")
  local props_done = true -- state:hasItem("task-props-done")
  local pit_done = false -- state:hasItem("task-orchestra-pit-done")
  local all_done = practise_done and backdrop_done and lighting_done and props_done
  return {
    practise = practise_done,
    backdrop = backdrop_done,
    lighting = lighting_done,
    props = props_done,
    pit = pit_done,
    all = all_done,
  }
end

return {
  zones = {
    { -- todo list
      cursor = "eye",
      visible = function(state) return state:hasItem("stage-has-list") end,
      click = function(state, item)

        local done = doneState(state)

        local function doneOrNot(d) 
          if d then
            return "Done."
          else
            return "Not done."
          end
        end

        if done.all then
          startDialog(createOneLiner("Everything is done.\nI should see Mr. Castello.")) 
          return
        end

        local dialog = {
          {
            who = "you",
            "Okay, let's check my tasks...",
          },
        }
        table.insert(dialog, {
          who = "you",
          "Practise with Aubrey: " .. doneOrNot(done.practise),
        })
        table.insert(dialog, {
          who = "you",
          "Place the backdrop: " .. doneOrNot(done.backdrop),
        })
        table.insert(dialog, {
          who = "you",
          "Set up the lighting: " .. doneOrNot(done.lighting),
        })
        table.insert(dialog, {
          who = "you",
          "Place the props: " .. doneOrNot(done.props),
        })

        if not state:hasItem("locker-removed-padlock") then
          table.insert(dialog, {
            who = "you",
            "Closing the orchestra pit: " .. doneOrNot(false),
          })
        end

        startDialog(createDialog(dialog))
      end,
      visual = { "objects/todolist.png", x=-84, y=7 },
      hint = "Task List",
      1279, 366, 1264, 350, 1183, 399, 1229, 502, 1279, 473, 
    },
    { -- right exit
      cursor = "walk",
      visible = true,
      click = function() setScene("workshop") end,
      hint = "Workshop",
      1127, 236, 1209, 668, 1059, 670, 1037, 592, 
    },
    { -- left exit
      cursor = "walk",
      visible = true,
      click = function() setScene("requisites") end,
      hint = "Requisites",
      126, 192, 156, 653, 315, 593, 188, 167, 
    },
    { -- bottom exit
      cursor = "walk",
      visible = true,
      click = function() setScene("entryhall") end,
      hint = "Entry Hall",
      269, 664, 223, 717, 1019, 719, 979, 664, 
    },
    { -- castello
      cursor = "talk",
      visible = function(state) return state:hasItem("stage-has-castello") end,
      click = function(state)
        if doneState(state).all then
          startDialog("accident-aubrey")
        else
          startDialog(createOneLiner("Don't you have work to do?", "castello")) 
        end
      end,
      hint = "Mr. Castello",
      visual = { "objects/castello_pose.png", x=-35, y=-86 },
      53, 259, 65, 174, 143, 172, 151, 259, 176, 276, 212, 378, 201, 399, 186, 409, 157, 661, 178, 700, 172, 713, 68, 715, 41, 491, 50, 406, 25, 375, 23, 277, 
    },

    { -- backdrop footer
      cursor = "eye",
      visible = function(state) return not state:hasItem("stage-has-backdrop") end,
      click = function(state, item)
        if item == "backdrop" then
          state:giveItem("stage-has-backdrop")
          state:removeItem("backdrop")
          state:giveItem("mood")
          state:giveItem("task-backdrop-done")
          startDialog(createOneLiner("Okay, the backdrop is now in place.\nOne task less!")) 
        else
          startDialog(createOneLiner("This is where i need to place the backdrop.")) 
        end
      end,
      visual = { "objects/backdrop-place.png", x=-17, y=-16 },
      hint = "Place for the backdrop",
      691, 635, 679, 688, 1012, 690, 1001, 641, 
    },
    { -- backdrop
      cursor = "eye",
      visible = function(state) return state:hasItem("stage-has-backdrop") end,
      click = function(state)
        startDialog(createOneLiner("This is the backdrop i made.")) 
      end,
      hint = "Backdrop",
      visual = { "objects/backdrop.png", x=0, y=0 },
      697, 185, 999, 182, 1000, 671, 698, 672,
    },
  }
}